@@ -108,8 +108,27 @@ class OpenStackAuthFederatedTestsMixin(object):
         client.federation.projects = self.mox.CreateMockAnything()
         client.federation.projects.list().AndReturn(projects)
 
+    def _mock_unscoped_list_domains(self, client, domains):
+        client.auth = self.mox.CreateMockAnything()
+        client.auth.domains().AndReturn(domains)
+
     def _mock_unscoped_token_client(self, unscoped, auth_url=None,
-                                    client=True):
+                                    client=True, plugin=None):
+        if not auth_url:
+            auth_url = settings.OPENSTACK_KEYSTONE_URL
+        if unscoped and not plugin:
+            plugin = self._create_token_auth(
+                None,
+                token=unscoped.auth_token,
+                url=auth_url)
+            plugin.get_access(mox.IsA(session.Session)).AndReturn(unscoped)
+        plugin.auth_url = auth_url
+        if client:
+            return self.ks_client_module.Client(
+                session=mox.IsA(session.Session),
+                auth=plugin)
+
+    def _mock_plugin(self, unscoped, auth_url=None):
         if not auth_url:
             auth_url = settings.OPENSTACK_KEYSTONE_URL
         plugin = self._create_token_auth(
@@ -117,16 +136,17 @@ class OpenStackAuthFederatedTestsMixin(object):
             token=unscoped.auth_token,
             url=auth_url)
         plugin.get_access(mox.IsA(session.Session)).AndReturn(unscoped)
-        plugin.auth_url = auth_url
-        if client:
-            return self.ks_client_module.Client(
-                session=mox.IsA(session.Session),
-                auth=plugin)
+        plugin.auth_url = settings.OPENSTACK_KEYSTONE_URL
+        return plugin
 
-    def _mock_federated_client_list_projects(self, unscoped, projects):
-        client = self._mock_unscoped_token_client(unscoped)
+    def _mock_federated_client_list_projects(self, unscoped_auth, projects):
+        client = self._mock_unscoped_token_client(None, plugin=unscoped_auth)
         self._mock_unscoped_federated_list_projects(client, projects)
 
+    def _mock_federated_client_list_domains(self, unscoped_auth, domains):
+        client = self._mock_unscoped_token_client(None, plugin=unscoped_auth)
+        self._mock_unscoped_list_domains(client, domains)
+
 
 class OpenStackAuthTestsV2(OpenStackAuthTestsMixin, test.TestCase):
 
@@ -478,6 +498,13 @@ class OpenStackAuthTestsV3(OpenStackAuthTestsMixin,
         self._mock_unscoped_list_projects_fail(client, user)
 
     def _mock_unscoped_list_projects_fail(self, client, user):
+        plugin = self._create_token_auth(
+            project_id=None,
+            domain_name=DEFAULT_DOMAIN,
+            token=self.data.unscoped_access_info.auth_token,
+            url=settings.OPENSTACK_KEYSTONE_URL)
+        plugin.get_access(mox.IsA(session.Session)).AndReturn(
+            self.data.domain_scoped_access_info)
         client.projects = self.mox.CreateMockAnything()
         client.projects.list(user=user.id).AndRaise(
             keystone_exceptions.AuthorizationFailure)
@@ -878,6 +905,7 @@ class OpenStackAuthTestsV3(OpenStackAuthTestsMixin,
         self.data = data_v3.generate_test_data(service_providers=True)
         self.sp_data = data_v3.generate_test_data(endpoint='http://sp2')
         projects = [self.data.project_one, self.data.project_two]
+        domains = []
         user = self.data.user
         unscoped = self.data.unscoped_access_info
         form_data = self.get_form_data(user)
@@ -918,7 +946,13 @@ class OpenStackAuthTestsV3(OpenStackAuthTestsMixin,
         # mock authenticate for service provider
         sp_projects = [self.sp_data.project_one, self.sp_data.project_two]
         sp_unscoped = self.sp_data.federated_unscoped_access_info
-        client = self._mock_unscoped_token_client(sp_unscoped, plugin.auth_url)
+        sp_unscoped_auth = self._mock_plugin(sp_unscoped,
+                                             auth_url=plugin.auth_url)
+        client = self._mock_unscoped_token_client(None, plugin.auth_url,
+                                                  plugin=sp_unscoped_auth)
+        self._mock_unscoped_list_domains(client, domains)
+        client = self._mock_unscoped_token_client(None, plugin.auth_url,
+                                                  plugin=sp_unscoped_auth)
         self._mock_unscoped_federated_list_projects(client, sp_projects)
         self._mock_scoped_client_for_tenant(sp_unscoped,
                                             self.sp_data.project_one.id,
@@ -954,6 +988,7 @@ class OpenStackAuthTestsV3(OpenStackAuthTestsMixin,
         self.data = data_v3.generate_test_data(service_providers=True)
         keystone_provider = 'localkeystone'
         projects = [self.data.project_one, self.data.project_two]
+        domains = []
         user = self.data.user
         unscoped = self.data.unscoped_access_info
         form_data = self.get_form_data(user)
@@ -964,7 +999,12 @@ class OpenStackAuthTestsV3(OpenStackAuthTestsMixin,
         self._mock_unscoped_token_client(unscoped,
                                          auth_url=auth_url,
                                          client=False)
-        client = self._mock_unscoped_token_client(unscoped, auth_url)
+        unscoped_auth = self._mock_plugin(unscoped)
+        client = self._mock_unscoped_token_client(None, auth_url=auth_url,
+                                                  plugin=unscoped_auth)
+        self._mock_unscoped_list_domains(client, domains)
+        client = self._mock_unscoped_token_client(None, auth_url=auth_url,
+                                                  plugin=unscoped_auth)
         self._mock_unscoped_list_projects(client, user, projects)
         self._mock_scoped_client_for_tenant(unscoped, self.data.project_one.id)
 
@@ -1051,6 +1091,32 @@ class OpenStackAuthTestsV3(OpenStackAuthTestsMixin,
             token=unscoped.auth_token)
         self.assertEqual(project_list, expected_projects)
 
+    def test_login_form_multidomain(self):
+        override = self.settings(OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT=True)
+        override.enable()
+        self.addCleanup(override.disable)
+
+        url = reverse('login')
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'name="domain" type="text"')
+
+    def test_login_form_multidomain_dropdown(self):
+        override = self.settings(OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT=True,
+                                 OPENSTACK_KEYSTONE_DOMAIN_DROPDOWN=True,
+                                 OPENSTACK_KEYSTONE_DOMAIN_CHOICES=(
+                                     ('Default', 'Default'),)
+                                 )
+        override.enable()
+        self.addCleanup(override.disable)
+
+        url = reverse('login')
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'select id="id_domain" name="domain"')
+        self.assertContains(response, 'option value="Default"')
+        settings.OPENSTACK_KEYSTONE_DOMAIN_DROPDOWN = False
+
 
 class OpenStackAuthTestsWebSSO(OpenStackAuthTestsMixin,
                                OpenStackAuthFederatedTestsMixin,
@@ -1147,11 +1213,14 @@ class OpenStackAuthTestsWebSSO(OpenStackAuthTestsMixin,
 
     def test_websso_login(self):
         projects = [self.data.project_one, self.data.project_two]
+        domains = []
         unscoped = self.data.federated_unscoped_access_info
         token = unscoped.auth_token
+        unscoped_auth = self._mock_plugin(unscoped)
 
         form_data = {'token': token}
-        self._mock_federated_client_list_projects(unscoped, projects)
+        self._mock_federated_client_list_domains(unscoped_auth, domains)
+        self._mock_federated_client_list_projects(unscoped_auth, projects)
         self._mock_scoped_client_for_tenant(unscoped, self.data.project_one.id)
 
         self.mox.ReplayAll()
@@ -1166,11 +1235,14 @@ class OpenStackAuthTestsWebSSO(OpenStackAuthTestsMixin,
         settings.OPENSTACK_KEYSTONE_URL = 'http://auth.openstack.org:5000/v3'
 
         projects = [self.data.project_one, self.data.project_two]
+        domains = []
         unscoped = self.data.federated_unscoped_access_info
         token = unscoped.auth_token
+        unscoped_auth = self._mock_plugin(unscoped)
 
         form_data = {'token': token}
-        self._mock_federated_client_list_projects(unscoped, projects)
+        self._mock_federated_client_list_domains(unscoped_auth, domains)
+        self._mock_federated_client_list_projects(unscoped_auth, projects)
         self._mock_scoped_client_for_tenant(unscoped, self.data.project_one.id)
 
         self.mox.ReplayAll()
